#-------------------------------------------------
#
# Title: Setup Software Packages & Extend State Boundaries in the USA 
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     November 2020; updated June 2022
#
# License:  MIT
#
# Description: Setup required packages for software;
#               download spatial layers and extend boundaries 
#               for the forty-eight conterminous United States
#
# Output: States48_buff_intersect_50km.gpkg
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
# To load the proper versions of each package, 
# un-comment lines 35-39, 49, 53, and 56 and run ONCE on your machine.

# NOTE for all packages below:
# Enter 3 (None) when prompted: These packages have more recent versions available. It is recommended to update all of them. Which would you like to update?

  # install.packages("devtools")
  # library(devtools)  
  # 
  # devtools::install_version("installr", version="0.23.2") ## install rtools
  # installr::install.Rtools()
  #
  # ## When prompted: "A newer R version is available, do you want to update R first?" Select: "No"
  # ## When prompted: "Do you want to allow this app to make changes to your device?" Select: "Yes"
  # ## Click "next", "next", "install", and "finish" when prompted.

  # ## to install fasterRaster package version 0.7.0 download zip file from:
  # ## https://github.com/adamlilith/fasterRaster/blob/master/zipTarFiles/fasterRaster_0.7.0.zip
  # ## Then install manually by clicking "Packages", "install package(s) from local files.", then navigating to the downloaded file, and clicking "open".
  #
  # devtools::install_version("renv", version="0.15.5") ## install renv
  # 
  # ## Once you've installed renv, ensure the working directory includes the renv.lock, .Rprofile, and renv/activate.R files that were included in the packet.
  # ## Then activate and restore the renv library - NOTE: this step will automatically install the package versions and dependencies necessary to run the software may take a while to complete.
  # renv::activate()
  # ## When prompted: "Do you want to proceed? [y/N]:" 
  # ## Type: y
  # renv::restore()
  # ## When prompted: "The following package(s) were installed from an unknown source:...."
  # ## Type: y
  # ## When prompted: "The following package(s) will be updated:...."
  # ## Type: y
  # 
  # ## If you receive an error message, make sure fasterRaster is installed (see lines 45-47 above) and run line 56 to restore the renv library again.

################################################################################
## ### NOTE: Only run the following if you wish to manually set up required packages or if activating the renv library fails
#  #
#  # devtools::install_version("sp", version="1.4-5")
#  # devtools::install_version("sf", version= "0.9-8")
#  # devtools::install_version("rgdal", version= "1.5.18")
#  # devtools::install_version("dplyr",  version= "1.0.6")
#  # devtools::install_version("tigris", version= "1.0")
#  # devtools::install_version("USAboundaries", version="0.4.0")
#  # devtools::install_version("raster", version= "3.5-9")
#  # devtools::install_version('terra', version="1.5-34")
#  # install.packages("XML")  # download most recent package from source (needed for rgrass7)
#  # install.packages("igraph") # download most recent package from source (needed for FedData)
#  # devtools::install_version("rgrass7", version= "0.2.7")
#  # devtools::install_version("foreach", version="1.5.1")
#  # devtools::install_version("FedData", version = "2.5.7")
#  # devtools::install_version("gdalUtils", version = "2.0.3.2")
#  # devtools::install_version("foreign", version = "0.8-80")
#  # devtools::install_version("plyr", version = "1.8.6")
#  # devtools::install_version("link2GI", version="0.4-5")
#  # devtools::install_version("rgeos", version="0.5-5")
################################################################################  
  
#### Load packages for this script
renv::restore()
library(sf)
library(rgdal)
library(tigris)

################################################################################

##### PROCESSING NOTES
### State boundaries are loaded using the tigris package which downloads TIGER/Line shapefiles 
### from the United States Census Bureau 
### (<https://www.census.gov/geo/maps-data/data/tiger-line.html>) 

############################################################################

options(tigris_use_cache=TRUE)  ### will cache Census shape file downloads to your machine
tigris_cache_dir((getwd())) ### set the cache directory to the current working directory
readRenviron('~/.Renviron') ### To use now, restart R or run `readRenviron('~/.Renviron')`
Sys.getenv('TIGRIS_CACHE_DIR') ### check that it was set

#### Download Cartographic Boundary Files with State outlines using tigris package
states<-states(cb = T, resolution = "500k", year = 2019, class="sf") ## cb=T downloads generalized states file, using class="sf" allows you to read in as a simple feature

st_crs(states) ## check CRS
summary(states)

### Remove the following states and territories 
states.remove<-c("District of Columbia","Guam", "Alaska","Puerto Rico","American Samoa",
  "United States Virgin Islands", "Hawaii","Commonwealth of the Northern Mariana Islands")
length(states$NAME)-length(states.remove)
states.48<-states
for(i in 1:length(states.remove)){
   states.48 <- states.48[states.48$NAME != (states.remove[i]),]
} 
length(states.48$NAME) ### should be 48

#### Create a 50 km buffer around each state to be used as a clip for spatial data inputs 
st_crs(states.48)  ### check CRS
States.pr<-st_transform(states.48, 2163) ### project to US National Atlas Equal Area (epsg = 2163)
States_buff<-st_buffer(States.pr, dist = 50000) ### add a 50 km buffer around each state polygon
States_buff_simp<-st_simplify(States_buff, dTolerance = 10000) ### simplify the outline of each polygon
plot(st_geometry(States_buff_simp))

st_crs(States_buff_simp)

### Remove all inner boundaries and return an outline of the US
US.pr <- st_transform(states, 2163)  ###  re-project the states layer with all states (otherwise will create a hole where DC is!)
US.outline <- st_union(US.pr)
plot(st_geometry(US.outline))
st_crs(US.outline)

### Intersect extended states with US outline to remove areas that extended beyond the national shoreline (over oceans/Great Lakes where no spatial data are present, i.e., NLCD, Soils, etc.)
States_buff_intersect<-st_intersection(States_buff_simp, US.outline)
plot(st_geometry(States_buff_intersect))

States_buff.pr<-st_transform(States_buff_intersect,st_crs(states)) ### transform back to original CRS

plot(States_buff.pr[States_buff.pr$STUSPS == "MD",][1]) ### plot MD to check

#### Write out extended state boundaries as a geopackage 
st_write(obj = States_buff.pr, dsn = "States48_buff_intersect_50km.gpkg", append = F)




