#-------------------------------------------------
#
# Title:    Select SPCS projections for the forty-eight conterminous United States  
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     December 2020; updated June 2022
#
# License:  MIT
#
# Description: Create csv file with projection information for each state for layer processing 
#
# Output: csv file with selected SPCS projections
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
###   State Plane Coordinate System in NAD83 was used for all states. 
### 	When a single SPCS zone was available for a state it was selected 
### 	Central was selected when a state was split into multiple SPCS zones (>2)
### 	North was selected when a state was split into north/south SPCS zones
### 	West was selected when a state was split into west/east SPCS zones 

#### Load packages
renv::restore()
library(rgdal)
library(sf)
library(USAboundaries)
library(tigris)

#### Download Cartographic Boundary Files with State outlines using tigris package
states<-states(cb = T, resolution = "500k", year = 2019, class="sf") ## cb=T downloads generalized states file, using class="sf" allows you to read in as a simple feature

### Remove the following states and territories 
states.remove<-c("District of Columbia","Guam", "Alaska","Puerto Rico","American Samoa",
                 "United States Virgin Islands", "Hawaii","Commonwealth of the Northern Mariana Islands")
length(states$NAME)-length(states.remove)
states.48<-states
for(i in 1:length(states.remove)){
  states.48 <- states.48[states.48$NAME != (states.remove[i]),]
} 
length(states.48$NAME) ### should be 48

states.df <- data.frame(states.48) ## convert to dataframe
states.df <- states.df[order(states.df[,'NAME']), ] ## order by state name
states.list <- data.frame(cbind(states.df$STUSPS, states.df$NAME)) ## save state codes and names as a df 
colnames(states.list) <- c("STUSPS", "NAME") ## rename columns
head(states.list)

#### NAD83 was used for raster manipulation in R 
#### Using the USAboundaries package and state_proj command can grab those EPSG codes

state.proj<-state_proj 

state.proj.keep<-state.proj[1,]
state.proj.statewide<-state.proj[state.proj$statewide_proj== T,]
stprsw<-state.proj.statewide[1,]
for(i in 1:length(states.list$STUSPS)){
  keep1 <- state.proj[state.proj$state == as.character(states.list$STUSPS[i]),]
  state.proj.keep<- rbind(state.proj.keep,keep1)
  keep2 <- state.proj.statewide[state.proj.statewide$state == as.character(states.list$STUSPS[i]),]
  stprsw <- rbind(stprsw, keep2)
}

state.proj.keep<-state.proj.keep[-1,] ## remove first row (duplicate)
stprsw<-stprsw[-1,] ## remove first row (duplicate)

stprsw ### Look to see which zones are available

### Single zone available: CT, DE, MD, MT, NE, NH, NJ, NC, RI, SC, TN, VT
### Central selected for: AZ, CO, ID, MI, MN, MO, NV, NM, NY, TX, UT, WI
### North selected for: AR, IA, KS, KY, LA, ND, OH, OK, OR, PA, SD, VA, WA, WV
### West selected for: AL, FL, GA, IL, IN, ME, MS
### Others: Mainland: MA, East Central: WY, Zone 3: CA

### update AL, FL, GA, IL, IN, LA, ME, and MS 
AL<-state.proj.keep[state.proj.keep$state == "AL",] ### Pull out AL
stprsw[1,]<-AL[2,] ### change to West
FL<-state.proj.keep[state.proj.keep$state == "FL",] ### Pull out FL
stprsw[8,]<-FL[2,] ### change FL to West
GA<-state.proj.keep[state.proj.keep$state == "GA",] ### Pull out GA
stprsw[9,]<-GA[2,] ### change to West
stprsw[11:20,]

IL<-state.proj.keep[state.proj.keep$state == "IL",] ### Pull out IL
stprsw[11,]<-IL[2,] ### change to West
IN<-state.proj.keep[state.proj.keep$state == "IN",] ### Pull out IN
stprsw[12,]<-IN[2,] ### change to West
LA<-state.proj.keep[state.proj.keep$state == "LA",] ### Pull out LA
stprsw[16,]<-LA[1,] ### change LA to North
ME<-state.proj.keep[state.proj.keep$state == "ME",] ### Pull out ME
stprsw[17,]<-ME[2,] ### change to West

stprsw[21:30,]
MS<-state.proj.keep[state.proj.keep$state == "MS",] ### Pull out MS
stprsw[22,]<-MS[2,] ### change to West

stprsw[31:40,]
stprsw[41:48,]

epsg.spcs <- stprsw

epsg.spcs$NAME <- states.list$NAME[match(epsg.spcs$state,  states.list$STUSPS)] ## Create new column with full names by matching codes across both df
epsg.df<-epsg.spcs[,c(1,6,2:4)] ### reorder columns 

colnames(epsg.df)<- c("STUSPS", "NAME","Zone", "EPSG_NAD83", "prj4_NAD83") ## rename columns
epsg.df

write.csv(epsg.df,"SPCS_EPSG_SOP4CWD.csv")  ### save the dataframe as csv for use in spatial data processing

#################################################