#-------------------------------------------------
#
# Title:    Process Distance to NHD Strahler4plus Streams in the USA  
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     February 2021; updated June 2022
#
# License:  MIT
#
# Description: Create a distance to Strahler order streams >=4 raster based on 
#              National Hydrography Dataset Plus Version 2.
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
### This script requires installation of Grass GIS 7.8
### Download from: https://grass.osgeo.org/download/
#
### Load libraries 
library(rgrass7)
grassDir <- 'C:/Program Files/GRASS GIS 7.8'  ### point R to the location of Grass on your computer
library(rgdal)
library(sf)
library(terra) 
library(raster)
use_sp()
library(fasterRaster) 

### NOTE: this script is dependent on the outputs from the:
###     1) "0_USA_Extend State Boundaries.R" script
###     2) "1_USA_Select SPCS Projections.R" script
###     3) "2_USA_Process NLCD Data.R" script for the state being processed.

### Inputs: 
###     1) gpkg of extended state boundaries created in the "0_USA_Extend State Boundaries.R" script 
###     2) csv with selected projections created in "1_USA_Select SPCS Projections.R" script
###     3) National Hydrology Dataset Plus Version 2 subset to include only Strahler order streams >=4 and named "NHDFlowline_Network_Strahler4plus.shp" 
###         *** This input layer must be downloaded and pre-processed externally before use in the script. ***
###         *** See Readme file Step 0 for download instructions and Step 17 for pre-processing directions. ***
###     4) Processed NLCD layer for the state created in "2_USA_Process NLCD Data.R" script used to define raster cell size, extent, and alignment
###     5) Sub folder for the state in your working directory created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

### Read in pre-processed hydrology layer with >=4 Strahler order streams from your working directory 
### *** see Readme file Steps 0 & 17 for detailed instructions ***
Water<-st_read("NHDFlowline_Network_Strahler4plus.shp")
### NOTE: this script can also be used to process distance rasters from other vector datasets. 
### To do so, change the code above to read in the desired vector dataset and change the file output names in the function below.

proj2<-st_crs(Water)

### Read in csv with selected projections created in "1_USA_Select SPCS Projections.R" script
spcs.proj<-read.csv("SPCS_EPSG_SOP4CWD.csv")
head(spcs.proj)
spcs.proj<-spcs.proj[,-1] ### remove the first column (no data)
spcs.proj$STUSPS

wd <- getwd() ## store working directory as an object
### NOTE: Your working directory must include the sub folder for the state being processed that was 
### created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

Process_dist_to_streams <- function(StatePostalCode){
  ### Set up working directory 
  StatePostalCode <- StatePostalCode
  setwd(wd)
  keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## save all to state sub folder created in NLCD script: StatePostalCode_EPSG
  
  ## Read in extended states geopackage and reproject hydrology layer to SPCS
  St.sub <- st_read(sprintf("%s_%s_buff_50km.gpkg",StatePostalCode, keep$EPSG_NAD83))
  plot(st_geometry(St.sub))
  St.sub.buff <- st_buffer(St.sub, dist = 10000) ### add a 10 km buffer around extended state for use in creating distance to streams
  plot(st_geometry(St.sub.buff), add=T)
  
  Water.pr <- st_transform(Water, crs=st_crs(St.sub))
  
  ## Crop the hydrology layer to the extended state outline (both with and without added buffer)
  St.water.buff <- st_intersection(st_zm(Water.pr), St.sub.buff)
  St.water <- st_intersection(st_zm(Water.pr), St.sub)
  plot(st_geometry(St.water.buff))
  plot(st_geometry(St.sub), add=T)
  
  ### Save cropped hydrology layer to directory
  st_write(obj = St.water, dsn = sprintf("Streams_NHDstrahler4plus_%s_%s.gpkg",StatePostalCode, keep$EPSG_NAD83), append = FALSE)
  
  ## Convert hydrology layer with buffer to a spatial object
  St.water.sp <- as_Spatial(St.water.buff)
  plot(St.water.sp, add=T, col="red")
  
  ### Read in processed NLCD layer to use as a template
  NLCD.st.pr<-raster(sprintf("NLCD2016_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
  
  ### Call on Grass using fasterRaster to calculate distance raster
  distToH2o <- fasterVectToRastDistance(rast=NLCD.st.pr, vect=St.water.sp, grassDir=grassDir) ## use water layer with 10 km buffer to avoid issues near edges of extent
  plot(distToH2o, main='Distance to Water (m)')
  plot(St.water.sp, col='blue', add=TRUE)
  raster::crs(distToH2o) <- CRS(SRS_string = sprintf("EPSG:%s",keep$EPSG_NAD83))
  
  ### Mask distance raster layer to extended state outline
  distTo.mask<-raster::mask(distToH2o,St.sub)
  
  ### Convert to spatraster
  distTo.mask.sr<-rast(distTo.mask)
 
  ### Write out the distance raster
  writeRaster(distTo.mask.sr, filename=sprintf("Streams_NHDstrahler4plus_distance_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2U',overwrite=TRUE)
  
  ### Set working directory back to main folder
  setwd(wd)  
}  

### Replace XX below with the 2 letter state postal code (for example, NY, OH, VA). 
### The line calling the function can be duplicated to run the process sequentially for multiple states.

# Process_dist_to_streams(StatePostalCode="XX") 

 
