#-------------------------------------------------
#
# Title:    Get DEM Layer for USA States
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     December 2020; updated June 2022
#
# License:  MIT
#
# Description: Download and process DEM layers for each state.
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
### load libraries 
renv::restore()
library(FedData)
library(terra) 
library(raster)
library(rgdal)
library(sp)

### NOTE: this script is dependent on the outputs from the:
###     1) "0_USA_Extend State Boundaries.R" script
###     2) "1_USA_Select SPCS Projections.R" script
###     3) "2_USA_Process NLCD Data.R" script for the state being processed.

### Inputs: 
###     1) gpkg of extended states created in the "0_USA_Extend State Boundaries.R" script
###     2) csv with selected projections created in "1_USA_Select SPCS Projections.R" script
###     3) FedData package to extract DEM tiles from the National Elevation Dataset
###     4) Processed NLCD layer for the state created in "2_USA_Process NLCD Data.R" script used to define raster cell size, extent, and alignment
###     5) Sub folder for the state in your working directory created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

### Multiple states didn't pull data for the entire extent (DE, IA, IL, MO, MD, PA, NJ) or returned an error message when processing (NY, VT). 
### These states were missing one or more tiles which were manually downloaded using modified NED_FUNCTIONS from FedData package 
### and then mosaicked, see "4_2_USA_Mosaic Missing DEM Tiles.R"

## Read in gpkg of extended states created in the "0_USA_Extend State Boundaries.R" script 
states.spdf<-readOGR("States48_buff_intersect_50km.gpkg")
plot(states.spdf)
proj4string(states.spdf)

### Read in csv with selected projections created in "1_USA_Select SPCS Projections.R" script
spcs.proj<-read.csv("SPCS_EPSG_SOP4CWD.csv")
head(spcs.proj)
spcs.proj<-spcs.proj[,-1] ### remove the first column (no data)
spcs.proj$STUSPS

### Depending on available memory creating a mosaic of downloaded tiles for an entire state may not be possible with get_ned
### Larger states (>40 NED tiles) may be split into two sections, extracted, masked, and then mosaicked
### Smaller states (<40 NED tiles) may be done all at once (see Get_DEM_layer_SMstate function below)

wd = getwd() ## store working directory as an object
### NOTE: Your working directory must include the sub folder for the state being processed that was 
### created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

Get_DEM_layer_LRGstate <- function(StatePostalCode){
  setwd(wd)
  
  ## subset extended state outline based on StatePostalCode
  St.sub <- subset(states.spdf, STUSPS == StatePostalCode) 
  plot(St.sub)
  
  ## use extent of the extended state outline to create two polygons
  e <- extent(St.sub)
  xmn=e[1] ; xmx= e[2] ; ymn=e[3] ; ymx=e[4]
  e.5 <- e
  ### some states might need a slight buffer on one edge, add to all sides 
  e.5[1] <- xmx-(xmx-xmn)*1.05
  e.5[2] <- xmn+(xmx-xmn)*1.05
  e.5[3] <- ymx-(ymx-ymn)*1.05
  e.5[4] <- ymn+(ymx-ymn)*1.05 
  ## divide into north and south by changing ymin and ymax
  e1 <- e.5 ; e1[3] <- ymn+(ymx-ymn)/2  
  e2 <- e.5 ; e2[4] <- ymn+(ymx-ymn)/2
  e1.poly<- polygon_from_extent(raster::extent(e1), proj4string = proj4string(St.sub))
  e2.poly<- polygon_from_extent(raster::extent(e2), proj4string = proj4string(St.sub))
  plot(e1.poly, add=T,col="green")
  plot(e2.poly, add=T, col="blue")
  plot(St.sub, add=T)
  
  ## Extract dem layer from the NED dataset for each polygon
  keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## save all to folder created in NLCD script: StatePostalCode_EPSG
  st.folder = getwd()
  raw.dir1 = paste(st.folder,sprintf("rawNED_%sN",StatePostalCode), sep= '/')
  extraction.dir1 = paste(st.folder,sprintf('extractionNED_%sN',StatePostalCode),sep='/')
  raw.dir2 = paste(st.folder,sprintf("rawNED_%sS",StatePostalCode), sep= '/')
  extraction.dir2 = paste(st.folder,sprintf('extractionNED_%sS',StatePostalCode),sep='/')
  DEM_north <- get_ned(template = e1.poly, label = sprintf("%s_DEM_N",StatePostalCode),
                       raw.dir = raw.dir1, extraction.dir = extraction.dir1)
  DEM_south <- get_ned(template = e2.poly, label = sprintf("%s_DEM_S",StatePostalCode),
                       raw.dir = raw.dir2, extraction.dir = extraction.dir2)
  
  ## Convert to spatraster
  N.sr <- rast(DEM_north)
  S.sr <- rast(DEM_south)
  
  ## Mosaic masked N and S DEM layers
  m.sr <- terra::mosaic(N.sr, S.sr, fun="mean")
  
  ## Reproject to SPCS
  ### Read in processed NLCD layer to use as a template
  NLCD.sr<-rast(sprintf("NLCD2016_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
  m.pr<-terra::project(m.sr, NLCD.sr, method="bilinear")
  
  ## Convert to raster layer to mask
  m.pr.rast <- raster::raster(m.pr)
  St.sub.pr <- spTransform(St.sub, crs(NLCD.sr))
  m.mask <- raster::mask(m.pr.rast, St.sub.pr)
  
  ## save files to folder created in NLCD script: StatePostalCode_EPSG
  m.mask.sr <- terra::rast(m.mask)
  plot(m.mask.sr)
  writeRaster(m.mask.sr,filename=sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2S',overwrite=TRUE) ### INT2S allows negative values, which coastal states have
  
  ## set working directory back to main folder
  setwd(wd)
}



Get_DEM_layer_SMstate <- function(StatePostalCode){
  setwd(wd)
  
  ## subset extended state oultline based on StatePostalCode
  St.sub <- subset(states.spdf, STUSPS == StatePostalCode) 
  plot(St.sub)
  
  ## Extract dem layer from the NED dataset for each polygon
  wd = getwd()
  keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## save all to folder created in NLCD script: StatePostalCode_EPSG
  st.folder = getwd()
  raw.dir = paste(st.folder,sprintf("rawNED_%s",StatePostalCode), sep= '/') 
  extraction.dir = paste(st.folder,sprintf('extractionNED_%s',StatePostalCode), sep='/')
  e <- extent(St.sub)
  xmn=e[1] ; xmx= e[2] ; ymn=e[3] ; ymx=e[4]  
  e4 <- e 
  ### some states might need a slight buffer on one edge, add to all sides 
  e4[1] <- xmx-(xmx-xmn)*1.05
  e4[2] <- xmn+(xmx-xmn)*1.05
  e4[3] <- ymx-(ymx-ymn)*1.05
  e4[4] <- ymn+(ymx-ymn)*1.05 
  e.poly2<- polygon_from_extent(raster::extent(e4), proj4string = proj4string(St.sub))
  plot(e.poly2)
  plot(St.sub, add=T)
  DEM <- get_ned(template = e.poly2, label = sprintf("%s_DEM",StatePostalCode),
                 raw.dir = raw.dir, extraction.dir = extraction.dir)
  
  ## Convert to spatraster
  DEM.sr <- rast(DEM)
  
  ## Reproject to SPCS
  ### Read in processed NLCD layer to use as a template
  NLCD.sr<-rast(sprintf("NLCD2016_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
  
  DEM.pr<-terra::project(DEM.sr, NLCD.sr, method="bilinear")
  
  ## Convert to raster layer to mask
  DEM.pr.rast <- raster::raster(DEM.pr)
  St.sub.pr <- spTransform(St.sub, crs(NLCD.sr))
  DEM.mask <- raster::mask(DEM.pr.rast, St.sub.pr)
  
  ## save files to folder created in NLCD script: StatePostalCode_EPSG
  DEM.mask.sr <- terra::rast(DEM.mask)
  plot(DEM.mask.sr)
  
  ## save dem to folder 
  writeRaster(DEM.mask.sr,filename=sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2S',overwrite=TRUE)
  
  ## set working directory back
  setwd(wd)
}

### Replace XX below with the 2 letter state postal code (for example, NY, OH, VA). 
### The line calling the function can be duplicated to run the process sequentially for multiple states.

#Get_DEM_layer_LRGstate(StatePostalCode="XX")

#Get_DEM_layer_SMstate(StatePostalCode="XX")  

