#-------------------------------------------------
#
# Title:    Mosaic Missing DEM Tiles in the USA 
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     December 2020; updated March 2022
#
# License:  MIT
#
# Description: Multiple states didn't pull DEM data for the entire extent (DE, IA, IL, MO, MD, PA, NJ) or returned an error message when processing (NY, VT). 
#              These states were missing one or more tiles which were manually downloaded using modified NED_FUNCTIONS from FedData package 
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
# Load packages
renv::restore()
library(FedData)
library(terra)
library(raster)
library(rgdal)
library(sp)
library(foreach)

### Note: This script is dependent on the outputs from the "4_1_USA_Get DEM Layer.R" script for these states.

### Multiple didn't pull DEM data for the entire extent: DE, IA, IL, MO, MD, PA, NJ, NY, VT. 
### These states were missing one or more tiles or produced errors during processing. Missing tiles were manually downloaded using code
### and modified NED_FUNCTIONS from the FedData package and then mosaicked with the statewide DEM layer created in "4_1_USA_Get DEM Layer.R". 
### We acknowledge use of code from the page: https://github.com/ropensci/FedData/blob/master/R/NED_FUNCTIONS.R

## Read in gpkg of extended states created in the "0_USA_Extend State Boundaries.R" script
states.spdf<-readOGR("States48_buff_intersect_50km.gpkg")
plot(states.spdf)
proj4string(states.spdf)

### Read in csv with selected projections created in "1_USA_Select SPCS Projections.R" script
spcs.proj<-read.csv("SPCS_EPSG_SOP4CWD.csv")
head(spcs.proj)
spcs.proj<-spcs.proj[,-1] ### remove the first column (no data)
spcs.proj$STUSPS

wd <- getwd() ## store working directory as an object
### NOTE: Your working directory must include the sub folder for the state being processed that was 
### created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

#### Set up working directory and other global variables 
Set_directory <- function(StatePostalCode){
  StatePostalCode <<- StatePostalCode
  setwd(wd)
  keep <<- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## save all to folder created in NLCD script: StatePostalCode_EPSG
  st.folder = getwd()
  raw.dir <<- paste(st.folder,sprintf("rawNED_%s",StatePostalCode), sep= '/')
}

### Modified NED_FUNCTIONS from FedData package to download desired arcgrid, unzip, and read in as raster
  missing_NED_download <-function(res = "1", tileNorthing, tileWesting, raw.dir){
  tmpdir <- tempfile()
  destdir <- paste(raw.dir, "/missing_tile", sep = "")
  dir.create(destdir, showWarnings = FALSE, recursive = TRUE)
  tileWesting <- formatC(tileWesting, width = 3, format = "d", flag = "0")
  tileNorthing <- formatC(tileNorthing, width = 2, format = "d", flag = "0")
  
  url <- paste0("https://prd-tnm.s3.amazonaws.com/StagedProducts/Elevation/", res, "/ArcGrid/USGS_NED_",res,"_n", tileNorthing, "w", tileWesting, 
                "_ArcGrid.zip")
  
  # If the new URL doesn't exist, try the old one.
  if(httr::http_error(url))
    url <- paste0("https://prd-tnm.s3.amazonaws.com/StagedProducts/Elevation/", res, "/ArcGrid/n", tileNorthing, "w", tileWesting, 
                  ".zip")
  file <- download_data(url = url, destdir = destdir)
  utils::unzip(file, exdir = tmpdir)
  dirs <- list.dirs(tmpdir, full.names = TRUE, recursive = F)
  dirs <- paste(dirs,sprintf("grdn%sw%s_1/w001001.adf",tileNorthing, tileWesting),sep="/")
  tile <- raster::raster(dirs)
  return(tile)
}

Project_mosaic_tile <- function(StatePostalCode, Tile){
  St.sub <- subset(states.spdf, STUSPS == StatePostalCode) 
  keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  DEM.sr <- rast(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
  
  ## Convert to spatraster and reproject
  Tile.sr <- rast(Tile)
  Tile.pr<-terra::project(Tile.sr, DEM.sr, method="bilinear")
  plot(DEM.sr)
  plot(Tile.pr, add=T)
  
  ## Mosaic masked N and S DEM layers
  m.sr <- terra::mosaic(Tile.pr, DEM.sr, fun="mean")
  
  ## Convert to raster layer to mask
  m.rast <- raster::raster(m.sr)
  St.sub.pr <- spTransform(St.sub, sprintf("+init=epsg:%s",keep$EPSG_NAD83))
  DEM.mask <- raster::mask(m.rast, St.sub.pr)

  ## save files to folder created in NLCD script: StatePostalCode_EPSG
  DEM.mask.sr <- terra::rast(DEM.mask)
  plot(DEM.mask.sr)
  writeRaster(DEM.mask.sr,filename=sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83),datatype='INT2S',overwrite=TRUE)
  setwd(wd) ## set working directory back to main folder
}

#########################
##### DE fix ########
#########################
Set_directory("DE")
DE.DEM <- raster(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
plot(DE.DEM)
#### Missing NED tile n41w76
DE.tile <- missing_NED_download(tileNorthing = 41, tileWesting = 76, raw.dir = raw.dir)
plot(DE.tile)
Project_mosaic_tile("DE",DE.tile)

##########################
###### IA Fix ##########
##########################
Set_directory("IA")

#### Missing NED tile n42w92
IA.tile <- missing_NED_download(tileNorthing = 42, tileWesting = 92, raw.dir = raw.dir)
plot(IA.tile)
Project_mosaic_tile("IA",IA.tile)

##########################
###### IL Fix ##########
##########################
Set_directory("IL")

#### Missing NED tile n42w92 (same as IA)
IL.tile <- missing_NED_download(tileNorthing = 42, tileWesting = 92, raw.dir = raw.dir)
plot(IL.tile)
Project_mosaic_tile("IL",IL.tile)

##########################
###### MO Fix ##########
##########################
Set_directory("MO")

#### Missing NED tile n42w92 (same as IA)
MO.tile <- missing_NED_download(tileNorthing = 42, tileWesting = 92, raw.dir = raw.dir)
plot(MO.tile)
Project_mosaic_tile("MO",MO.tile)

#########################
##### MD fix ########
#########################
Set_directory("MD")
MD.DEM <- raster(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
plot(MD.DEM)

#### Missing NED tile n41w76 (same as DE)
MD.tile <- missing_NED_download(tileNorthing = 41, tileWesting = 76, raw.dir = raw.dir)
plot(MD.tile)
Project_mosaic_tile("MD",MD.tile)

#########################
##### PA fix ############
#########################
Set_directory("PA")
PA.DEM <- raster(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
plot(PA.DEM)

#### Missing NED tile n41w76 (same as DE)
PA.tile <- missing_NED_download(tileNorthing = 41, tileWesting = 76, raw.dir = raw.dir)
plot(PA.tile)
Project_mosaic_tile("PA",PA.tile)

#########################
##### NJ fix ############
#########################
Set_directory("NJ")
NJ.DEM <- raster(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
plot(NJ.DEM)

#### Missing NED tile n41w76 (same as DE)
NJ.tile <- missing_NED_download(tileNorthing = 41, tileWesting = 76, raw.dir = raw.dir)
plot(NJ.tile)
Project_mosaic_tile("NJ",NJ.tile)

########################
###### NY fix ##########
########################
### Received errors for NY with Get_DEM_layer function due to an issue with one tile. 
### Code to manually create DEM below

StatePostalCode = "NY"

St.sub <- subset(states.spdf, STUSPS == StatePostalCode) 
plot(St.sub)

## use extent of the extended state outline to create two polygons
e <- extent(St.sub)
xmn=e[1] ; xmx= e[2] ; ymn=e[3] ; ymx=e[4]
e.5 <- e
### some states might need a slight buffer on one edge, add to all sides 
e.5[1] <- xmx-(xmx-xmn)*1.05
e.5[2] <- xmn+(xmx-xmn)*1.05
e.5[3] <- ymx-(ymx-ymn)*1.05
e.5[4] <- ymn+(ymx-ymn)*1.05 
## divide into north and south by changing ymin and ymax
e1 <- e.5 ; e1[3] <- ymn+(ymx-ymn)/2  
e2 <- e.5 ; e2[4] <- ymn+(ymx-ymn)/2
e1.poly<- polygon_from_extent(raster::extent(e1), proj4string = proj4string(St.sub))
e2.poly<- polygon_from_extent(raster::extent(e2), proj4string = proj4string(St.sub))
plot(e1.poly, add=T,col="green")
plot(e2.poly, add=T, col="blue")
plot(St.sub, add=T)

## Extract dem layer from the NED dataset for each polygon
wd = getwd()
keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## save all to folder created in NLCD script: StatePostalCode_EPSG
st.folder = getwd()
raw.dir1 = paste(st.folder,sprintf("rawNED_%sN",StatePostalCode), sep= '/')
extraction.dir1 = paste(st.folder,sprintf('extractionNED_%sN',StatePostalCode),sep='/')
raw.dir2 = paste(st.folder,sprintf("rawNED_%sS",StatePostalCode), sep= '/')
extraction.dir2 = paste(st.folder,sprintf('extractionNED_%sS',StatePostalCode),sep='/')

### Southern portion of the state works, Northern gives an error 
DEM_south <- get_ned(template = e2.poly, label = sprintf("%s_DEM_S",StatePostalCode),
                     raw.dir = raw.dir2, extraction.dir = extraction.dir2)

#### Northern fix 
e1.poly.clip<-crop(e1.poly, St.sub)
plot(e1.poly.clip, add=T)
raw.dir = raw.dir1
extraction.dir = extraction.dir1
template = e1.poly.clip  
label = sprintf("%s_DEM_N",StatePostalCode)
res = 1
force.redo = F
raster.options = c("COMPRESS=DEFLATE",
                   "ZLEVEL=9",
                   "INTERLEAVE=BAND")
raw.dir <- normalizePath(paste0(raw.dir, "/."), mustWork = FALSE)
extraction.dir <- normalizePath(paste0(extraction.dir, "/."), mustWork = FALSE)

dir.create(raw.dir, showWarnings = FALSE, recursive = TRUE)
dir.create(extraction.dir, showWarnings = FALSE, recursive = TRUE)

template <- sp::spTransform(polygon_from_extent(e1.poly.clip), sp::CRS("+proj=longlat +ellps=WGS84"))
extent.latlon <- raster::extent(template)


if (file.exists(paste0(extraction.dir, "/", label, "_NED_", res, ".tif")) & !force.redo) {
  extracted.DEM <- raster::raster(paste0(extraction.dir, "/", label, "_NED_", res, ".tif"))
  return(extracted.DEM)
}

# Open USGS NED download service.  NED tiles are labeled by their northwest corner. Thus, coordinate 36.42N, -105.71W is in
# grid n37w106
wests <- seq(ceiling(abs(extent.latlon@xmax)), ceiling(abs(extent.latlon@xmin)))
norths <- seq(ceiling(abs(extent.latlon@ymin)), ceiling(abs(extent.latlon@ymax)))

tilesLocations <- as.matrix(expand.grid(norths, wests, stringsAsFactors = FALSE))

# Download and crop tiles
loc <- NULL
tiles <- foreach::foreach(loc = 1:nrow(tilesLocations)) %do% {
  return(tryCatch(get_ned_tile(
    template = template,
    res = res,
    tileNorthing = tilesLocations[loc, 1],
    tileWesting = tilesLocations[loc, 2],
    raw.dir = raw.dir
  ),
  error = function(e) {
    message("WARNING: ", e$message)
    return(NULL)
  },
  warning = function(w) NULL
  ))
}

tiles <- tiles[which(!sapply(tiles, is.null))]

tiles.keep <-tiles[c(1:10,12:32)]  ### 11th tile is the issue: n45w75

tiles.keep$fun <- mean
names(tiles.keep)[1:2] <- c("x", "y")
tiles.keep.m <- do.call(raster::mosaic, tiles.keep)
plot(tiles.keep.m)

DEM_north <- tiles.keep.m

## Convert to spatraster
N.sr <- rast(DEM_north)
S.sr <- rast(DEM_south)

## Mosaic masked N and S DEM layers
m.sr <- terra::mosaic(N.sr, S.sr, fun="mean")

## Reproject to SPCS
### Read in processed NLCD layer to use as a template
NLCD.sr<-rast(sprintf("NLCD2016_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
m.pr<-terra::project(m.sr, NLCD.sr, method="bilinear")

## Convert to raster layer to mask
m.pr.rast <- raster::raster(m.pr)
St.sub.pr <- spTransform(St.sub, crs(NLCD.sr))
m.mask <- raster::mask(m.pr.rast, St.sub.pr)

## save files to folder created in NLCD script: StatePostalCode_EPSG
m.mask.sr <- terra::rast(m.mask)
plot(m.mask.sr)
writeRaster(m.mask.sr,filename=sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2S',overwrite=TRUE) ### INT2S allows negative values, which coastal states have

###########################
#### add the missing tile manually
Set_directory("NY")
NY.DEM <- rast(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
plot(NY.DEM)
#### Missing NED tile n45w75 
#NY.tile <- missing_NED_download(tileNorthing = 45, tileWesting = 75, raw.dir = raw.dir) ## error message, download manually below

tileWesting = 75
tileNorthing = 45

tmpdir <- tempfile()
destdir <- paste(raw.dir, "/missing_tile", sep = "")
dir.create(destdir, showWarnings = FALSE, recursive = TRUE)
tileWesting <- formatC(tileWesting, width = 3, format = "d", flag = "0")
tileNorthing <- formatC(tileNorthing, width = 2, format = "d", flag = "0")

url <- paste0("https://prd-tnm.s3.amazonaws.com/StagedProducts/Elevation/", res, "/ArcGrid/USGS_NED_",res,"_n", tileNorthing, "w", tileWesting, 
              "_ArcGrid.zip")

file <- download_data(url = url, destdir = destdir)
utils::unzip(file, exdir = tmpdir)
dirs <- list.dirs(tmpdir, full.names = TRUE, recursive = F)
dirs <- paste(dirs,sprintf("/w001001.adf",tileNorthing, tileWesting),sep="/")
tile <- raster::raster(dirs[1])
NY.tile <- tile

DEM.sr <- rast(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))

## Convert to spatraster and reproject
Tile.sr <- rast(NY.tile)
Tile.pr<-terra::project(Tile.sr, DEM.sr, method="bilinear")
plot(DEM.sr)
plot(Tile.pr, add=T)

## Mosaic masked N and S DEM layers
m.sr <- terra::mosaic(Tile.pr, DEM.sr, fun="mean")

## Convert to raster layer to mask
m.rast <- raster::raster(m.sr)
St.sub.pr <- spTransform(St.sub, sprintf("+init=epsg:%s",keep$EPSG_NAD83))
DEM.mask <- raster::mask(m.rast, St.sub.pr)

## save files to folder created in NLCD script: StatePostalCode_EPSG
DEM.mask.sr <- terra::rast(DEM.mask)
plot(DEM.mask.sr)
writeRaster(DEM.mask.sr,filename=sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83),datatype='INT2S',overwrite=TRUE)
setwd(wd)

#### Add additional missing NED tile n41w76 (same as NJ)
NY.tile <- missing_NED_download(tileNorthing = 41, tileWesting = 76, raw.dir = raw.dir)
plot(NY.tile)
Project_mosaic_tile("NY",NY.tile)

########################
###### VT fix ##########
########################
### Received errors for VT with Get_DEM_layer function due to an issue with one tile (same as with NY above). 
### Code to manually create DEM below

StatePostalCode = "VT"

St.sub <- subset(states.spdf, STUSPS == StatePostalCode) 
plot(St.sub)

## use extent of the extended state outline to create two polygons
e <- extent(St.sub)
xmn=e[1] ; xmx= e[2] ; ymn=e[3] ; ymx=e[4]
e.5 <- e
### some states might need a slight buffer on one edge, add to all sides 
e.5[1] <- xmx-(xmx-xmn)*1.05
e.5[2] <- xmn+(xmx-xmn)*1.05
e.5[3] <- ymx-(ymx-ymn)*1.05
e.5[4] <- ymn+(ymx-ymn)*1.05 
## divide into north and south by changing ymin and ymax
e1 <- e.5 ; e1[3] <- ymn+(ymx-ymn)/2  
e2 <- e.5 ; e2[4] <- ymn+(ymx-ymn)/2
e1.poly<- polygon_from_extent(raster::extent(e1), proj4string = proj4string(St.sub))
e2.poly<- polygon_from_extent(raster::extent(e2), proj4string = proj4string(St.sub))
plot(e1.poly, add=T,col="green")
plot(e2.poly, add=T, col="blue")
plot(St.sub, add=T)

## Extract dem layer from the NED dataset for each polygon
wd = getwd()
keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## save all to folder created in NLCD script: StatePostalCode_EPSG
st.folder = getwd()
raw.dir1 = paste(st.folder,sprintf("rawNED_%sN",StatePostalCode), sep= '/')
extraction.dir1 = paste(st.folder,sprintf('extractionNED_%sN',StatePostalCode),sep='/')
raw.dir2 = paste(st.folder,sprintf("rawNED_%sS",StatePostalCode), sep= '/')
extraction.dir2 = paste(st.folder,sprintf('extractionNED_%sS',StatePostalCode),sep='/')

### Southern portion of the state works, Northern gives an error 
DEM_south <- get_ned(template = e2.poly, label = sprintf("%s_DEM_S",StatePostalCode),
                     raw.dir = raw.dir2, extraction.dir = extraction.dir2)

#### Northern fix 
e1.poly.clip<-crop(e1.poly, St.sub)
plot(e1.poly.clip, add=T)
raw.dir = raw.dir1
extraction.dir = extraction.dir1
template = e1.poly.clip  
label = sprintf("%s_DEM_N",StatePostalCode)
res = 1
force.redo = F
raster.options = c("COMPRESS=DEFLATE",
                   "ZLEVEL=9",
                   "INTERLEAVE=BAND")
raw.dir <- normalizePath(paste0(raw.dir, "/."), mustWork = FALSE)
extraction.dir <- normalizePath(paste0(extraction.dir, "/."), mustWork = FALSE)

dir.create(raw.dir, showWarnings = FALSE, recursive = TRUE)
dir.create(extraction.dir, showWarnings = FALSE, recursive = TRUE)

template <- sp::spTransform(polygon_from_extent(e1.poly.clip), sp::CRS("+proj=longlat +ellps=WGS84"))
extent.latlon <- raster::extent(template)


if (file.exists(paste0(extraction.dir, "/", label, "_NED_", res, ".tif")) & !force.redo) {
  extracted.DEM <- raster::raster(paste0(extraction.dir, "/", label, "_NED_", res, ".tif"))
  return(extracted.DEM)
}

# Open USGS NED download service.  NED tiles are labeled by their northwest corner. Thus, coordinate 36.42N, -105.71W is in
# grid n37w106
wests <- seq(ceiling(abs(extent.latlon@xmax)), ceiling(abs(extent.latlon@xmin)))
norths <- seq(ceiling(abs(extent.latlon@ymin)), ceiling(abs(extent.latlon@ymax)))

tilesLocations <- as.matrix(expand.grid(norths, wests, stringsAsFactors = FALSE))

# Download and crop tiles
loc <- NULL
tiles <- foreach::foreach(loc = 1:nrow(tilesLocations)) %do% {
  return(tryCatch(get_ned_tile(
    template = template,
    res = res,
    tileNorthing = tilesLocations[loc, 1],
    tileWesting = tilesLocations[loc, 2],
    raw.dir = raw.dir
  ),
  error = function(e) {
    message("WARNING: ", e$message)
    return(NULL)
  },
  warning = function(w) NULL
  ))
}

tiles <- tiles[which(!sapply(tiles, is.null))]

tiles.keep <-tiles[c(1:13,15)]  ### 14th tile is the issue: n45w75

tiles.keep$fun <- mean
names(tiles.keep)[1:2] <- c("x", "y")
tiles.keep.m <- do.call(raster::mosaic, tiles.keep)
plot(tiles.keep.m)

DEM_north <- tiles.keep.m

## Convert to spatraster
N.sr <- rast(DEM_north)
S.sr <- rast(DEM_south)

## Mosaic masked N and S DEM layers
m.sr <- terra::mosaic(N.sr, S.sr, fun="mean")

## Reproject to SPCS
### Read in processed NLCD layer to use as a template
NLCD.sr<-rast(sprintf("NLCD2016_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
m.pr<-terra::project(m.sr, NLCD.sr, method="bilinear")

## Convert to raster layer to mask
m.pr.rast <- raster::raster(m.pr)
St.sub.pr <- spTransform(St.sub, crs(NLCD.sr))
m.mask <- raster::mask(m.pr.rast, St.sub.pr)

## save files to folder created in NLCD script: StatePostalCode_EPSG
m.mask.sr <- terra::rast(m.mask)
plot(m.mask.sr)
writeRaster(m.mask.sr,filename=sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2S',overwrite=TRUE) ### INT2S allows negative values, which coastal states have

###########################
#### add the missing tile manually
Set_directory("VT")
VT.DEM <- rast(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
plot(VT.DEM)
#### Missing NED tile n45w75 

tileWesting = 75
tileNorthing = 45

tmpdir <- tempfile()
destdir <- paste(raw.dir, "/missing_tile", sep = "")
dir.create(destdir, showWarnings = FALSE, recursive = TRUE)
tileWesting <- formatC(tileWesting, width = 3, format = "d", flag = "0")
tileNorthing <- formatC(tileNorthing, width = 2, format = "d", flag = "0")

url <- paste0("https://prd-tnm.s3.amazonaws.com/StagedProducts/Elevation/", res, "/ArcGrid/USGS_NED_",res,"_n", tileNorthing, "w", tileWesting, 
              "_ArcGrid.zip")

file <- download_data(url = url, destdir = destdir)
utils::unzip(file, exdir = tmpdir)
dirs <- list.dirs(tmpdir, full.names = TRUE, recursive = F)
dirs <- paste(dirs,sprintf("/w001001.adf",tileNorthing, tileWesting),sep="/")
tile <- raster::raster(dirs[1])
VT.tile <- tile

DEM.sr <- rast(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))

## Convert to spatraster and reproject
Tile.sr <- rast(VT.tile)
Tile.pr<-terra::project(Tile.sr, DEM.sr, method="bilinear")
plot(DEM.sr)
plot(Tile.pr, add=T)

## Mosaic masked N and S DEM layers
m.sr <- terra::mosaic(Tile.pr, DEM.sr, fun="mean")

## Convert to raster layer to mask
m.rast <- raster::raster(m.sr)
St.sub.pr <- spTransform(St.sub, sprintf("+init=epsg:%s",keep$EPSG_NAD83))
DEM.mask <- raster::mask(m.rast, St.sub.pr)

## save files to folder created in NLCD script: StatePostalCode_EPSG
DEM.mask.sr <- terra::rast(DEM.mask)
plot(DEM.mask.sr)
writeRaster(DEM.mask.sr,filename=sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83),datatype='INT2S',overwrite=TRUE)
setwd(wd)
