#-------------------------------------------------
#
# Title:    Create Slope & Roughness Layers 
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     December 2020; updated June 2022
#
# License:  MIT
#
# Description: Create slope and roughness layers for each state from processed DEM.
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
### load packages
renv::restore()
library(terra)
library(raster)
library(rgdal)

### NOTE: this script is dependent on the outputs from the:
###     1) "0_USA_Extend State Boundaries.R" script
###     2) "1_USA_Select SPCS Projections.R" script
###     3) "2_USA_Process NLCD Data.R" script for the state being processed
###     4) "4_1_USA_Get DEM Layer.R"/"4_2_USA_Mosaic Missing DEM Tiles.R" script for the state being processed.

### Inputs: 
###     1) gpkg of extended state boundaries created in the "0_USA_Extend State Boundaries.R" script 
###     2) csv with selected projections created in "1_USA_Select SPCS Projections.R" script
###     3) DEM layer projected and masked to the selected state created in "4_1_USA_Get DEM Layer.R"/"4_2_USA_Mosaic Missing DEM Tiles.R" script
###     4) Sub folder for the state in your working directory created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

### Read in csv with selected projections created in "1_USA_Select SPCS Projections.R" script
spcs.proj<-read.csv("SPCS_EPSG_SOP4CWD.csv")
head(spcs.proj)
spcs.proj<-spcs.proj[,-1] ### remove the first column (no data)

wd <- getwd() ## store working directory as an object
### NOTE: Your working directory must include the sub folder for the state being processed that was 
### created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

Create_SlopeRough_rasts <- function(StatePostalCode){
  setwd(wd)
  keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## set working directory to sub folder created in "2_USA_Process NLCD script" (i.e., StatePostalCode_EPSG)
  
  St.sub.pr <- readOGR(sprintf("%s_%s_buff_50km.gpkg",StatePostalCode, keep$EPSG_NAD83)) ## read in gpkg of extended state boundary
  DEM <- rast(sprintf("DEM_elevation_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83)) ## read in DEM layer
  
  slope_rast <- terrain(DEM, v="slope", unit="degrees") ## create slope raster
  rough_rast <- terrain(DEM, v="roughness") ## create roughness raster
  plot(slope_rast) ## plot to check!
  plot(St.sub.pr, add=T) ## plot to check!
  
  ### write out both rasters
  writeRaster(slope_rast,filename=sprintf("DEM_slope_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2S',overwrite=TRUE)
  writeRaster(rough_rast,filename=sprintf("DEM_roughness_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2S',overwrite=TRUE)
  setwd(wd) ### set working directory back to main folder
}

### Replace XX below with the 2 letter state postal code (for example, NY, OH, VA). 
### The line calling the function can be duplicated to run the process sequentially for multiple states.

#Create_SlopeRough_rasts(StatePostalCode="XX")  
