#-------------------------------------------------
#
# Title:    Process gNATSGO Average Clay Layer 
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     February 2021; updated June 2022
# 
# License:  MIT
#
# Description: Process % clay layer for each state
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
## Load libraries 
renv::restore()
library(plyr)
library(terra) 
library(raster)
library(rgdal)

### NOTE: this script is dependent on the outputs from the:
###     1) "0_USA_Extend State Boundaries.R" script
###     2) "1_USA_Select SPCS Projections.R" script
###     3) "2_USA_Process NLCD Data.R" script for the state being processed.

### Inputs: 
###     1) gpkg of extended state boundaries created in the "0_USA_Extend State Boundaries.R" script and saved to state folder
###     2) csv with selected projections created in "1_USA_Select SPCS Projections.R" script
###     3) gNATSGO "gNATSGO-mukey.tif" raster layer extracted from gNATSGO_mukey_grid folder 
###         *** This input layer must be downloaded and pre-processed externally before use in the script. ***
###         *** See Readme file Step 0 for download instructions and Step 44 for pre-processing directions. ***
###     4) csv file with average % clay in the upper 50 cm of soil derived from gNATSGO ("mapunit_clay.csv")
###         *** This input layer must be downloaded and pre-processed externally before use in the script. ***
###         *** See Readme file Step 0 for download instructions and Step 44-45 for pre-processing directions. ***
###     5) Processed NLCD layer for the state created in "2_USA_Process NLCD Data.R" script used to define raster cell size, extent, and alignment
###     6) Sub folder for the state in your working directory created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

### Read in CONUS gNATSGO raster and csv with average clay calculated 
### *** see Readme file Steps 0, 44, & 45 for detailed instructions ***
natsgo.rast <- raster("gNATSGO-mukey.tif")
clay <- read.csv("mapunit_clay.csv")

## Pull out 'mukey' and 'AVG_CLAY' from text
head(clay)
str(clay)
clay[,1]<-as.numeric(clay[,1])
clay.cpy <- clay
names(clay.cpy) <- c("ID","AVG_CLAY") ## change 'mukey' to 'ID' for use later 
str(clay.cpy)

### Read in csv with selected projections created in "1_USA_Select SPCS Projections.R" script
spcs.proj<-read.csv("SPCS_EPSG_SOP4CWD.csv")
head(spcs.proj)
spcs.proj<-spcs.proj[,-1] ### remove the first column (no data)
spcs.proj$STUSPS

wd <- getwd() ## store working directory as an object
### NOTE: Your working directory must include the sub folder for the state being processed that was 
### created in "2_USA_Process NLCD Data.R" script with the naming format "StatePostalCode_EPSG" (e.g. "NY_32116")

Process_Clay_Layer <- function(StatePostalCode){
  ### Set up working directory 
  StatePostalCode <- StatePostalCode
  setwd(wd)
  keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83)) ## save all to folder created in NLCD script: StatePostalCode_EPSG
  
  ## Read in extended states geopackage and reproject to raster
  St.sub <- readOGR(sprintf("%s_%s_buff_50km.gpkg",StatePostalCode, keep$EPSG_NAD83))
  St.sub.pr <-spTransform(St.sub, proj4string(natsgo.rast))
  plot(St.sub.pr)
  
  ### Crop gNATSGO raster to state level 
  St.soils <- crop(natsgo.rast,St.sub.pr)
  St.soils <- mask(St.soils, St.sub.pr)
  
  ### Generate a raster attribute table (RAT) using ratify function and add the AVG_CLAY table based on the mukey field in both
  soil.rat <- ratify(St.soils, count=TRUE)  ## convert cropped soils layer into raster and RAT
  rat <- levels(soil.rat)[[1]] ## save RAT to new object
  rat.new <- join(rat, clay.cpy, type='left') ## join RAT with clay data using left join
  levels(soil.rat) <- rat.new ## insert modified RAT back into soil raster
  St.clay <- deratify(soil.rat, att='AVG_CLAY') ## convert back to raster based on AVG_CLAY
  
  ## Reproject to SPCS
  ### Read in processed NLCD layer to use as a template
  NLCD.st.sr<-rast(sprintf("NLCD2016_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83))
  
  ## Convert to spatraster
  St.clay.sr <- rast(St.clay)
  
  St.clay.pr <- terra::project(St.clay.sr, NLCD.st.sr, method="bilinear")
  
  plot(St.clay.pr) ## plot to check
  plot(St.sub,add=T)
  
  ## Save clay layer to folder 
  writeRaster(St.clay.pr,filename=sprintf("Soils_gNATSGO_30m_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT2U',overwrite=TRUE)
  
  setwd(wd)   ### Set working directory back 
}

### Replace XX below with the 2 letter state postal code (for example, NY, OH, VA). 
### The line calling the function can be duplicated to run the process sequentially for multiple states.

#Process_Clay_Layer(StatePostalCode="XX") 
