#-------------------------------------------------
#
# Title:    Process NLCD Data in the USA
#
# Authors:  Corey Mitchell
#           W. David Walter
#           Nicholas Hollingshead
#           Krysten Schuler
#
# Location: Cornell Wildlife Health Lab
#           Cornell University
#
# Date:     December 2020; updated June 2022
#
# License:  MIT
#
# Description: Reclassify landcover data for each state and save rasters 
#              representing forested, open, and developed landcover.
#-------------------------------------------------

# This code was written under 
# R version 4.0.2 (2020-06-22) -- "Taking Off Again"
# Copyright (C) 2020 The R Foundation for Statistical Computing
# Platform: x86_64-w64-mingw32/x64 (64-bit)
#
#### Load packages
renv::restore()
library(rgdal)
library(terra) 
library(raster)
library(sp)
library(sf)
library(tigris)

### NOTE: this script is dependent on the outputs from the:
###     1) "0_USA_Extend State Boundaries.R" script
###     2) "1_USA_Select SPCS Projections.R" script

### Inputs: 
###     1) gpkg of extended states created in the "0_USA_Extend State Boundaries.R" script
###     2) csv with selected projections created in "1_USA_Select SPCS Projections.R" script
###     3) National Land Cover Database (NLCD) 2016 Land Cover dataset for the conterminous United States (CONUS)
###           downloaded from: https://www.mrlc.gov/data/nlcd-2016-land-cover-conus 
###     4) tigris package to download U.S. Cartographic Boundary Files with state outlines

#### The following function will create a sub folder in your working directory to store processed files. 
#### This folder will then be used in subsequent R scripts to save processed files.
#### *** This script must be run for a state before subsequent scripts can be run. ***

#### The output from this function includes 4 raster files and 2 geopackage files
####    which will be saved to the new sub folder in your working directory:
####    1) NLCD layer extent limited to state area and projected to the selected SPCS
####    2) NLCD layer reclassified to forest cover and projected to the selected SPCS
####    3) NLCD layer reclassified to developed cover and projected to the selected SPCS
####    4) NLCD layer reclassified to open cover and projected to the selected SPCS
####    5) Geopackage of the extended state's extent projected to the selected SPCS
####    6) Geopackage of the state cartographic boundary projected to the selected SPCS

#### NOTE: Depending on the size of the state being processed, this script may take awhile to run. 

## Read in gpkg of extended states created in the "0_USA_Extend State Boundaries.R" script 
states.spdf<-readOGR("States48_buff_intersect_50km.gpkg")
plot(states.spdf)
proj4string(states.spdf)

### read in NLCD img file from the working directory 
## (Important: retain all files that were included in the downloaded/unzipped NLCD layer folder, it will cause an error if separated)
NLCD<-raster("nlcd_2016_land_cover_l48_20210604\\nlcd_2016_land_cover_l48_20210604.img")

### need to match projections 
states.pr<-spTransform(states.spdf, proj4string(NLCD))
plot(NLCD)
plot(states.pr, add=T)

### Read in csv with selected projections created in "1_USA_Select SPCS Projections.R" script
spcs.proj<-read.csv("SPCS_EPSG_SOP4CWD.csv")
head(spcs.proj)
spcs.proj<-spcs.proj[,-1] ### remove the first column (no data)
spcs.proj$STUSPS

#### Download Cartographic Boundary Files with state outlines using tigris package
options(tigris_use_cache=TRUE)  ### will cache Census shape file downloads to your machine
tigris_cache_dir((getwd())) ### set the cache directory to the current working directory
readRenviron('~/.Renviron') ### To use now, restart R or run `readRenviron('~/.Renviron')`
Sys.getenv('TIGRIS_CACHE_DIR') ### check that it was set
states.boundary<-states(cb = T, resolution = "500k", year = 2019, class="sp") ## cb=T downloads generalized states file, using class="sp allows you to read in as spatial

wd <- getwd() ## store working directory as an object

Process_NLCD_layers <- function(StatePostalCode){
  ## Subset extended state outline based on StatePostalCode
  St.sub <- subset(states.pr, STUSPS == StatePostalCode) 
  plot(St.sub)
  
  ## Crop and mask the NLCD layer to the extended state outline
  NLCD.crop <- raster::crop(NLCD,St.sub)
  NLCD.mask <- raster::mask(NLCD.crop,St.sub)
  plot(NLCD.mask, add=T)
  
  #### Create directory to save files to (will be used in all subsequent processing)
  setwd(wd)
  keep <- spcs.proj[spcs.proj$STUSPS == StatePostalCode,]
  dir.create(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83))
  setwd(paste0(getwd(),"/",StatePostalCode,"_",keep$EPSG_NAD83))
  
  ### Re-project geopackage of extended state and save to same directory
  a_crs <-sprintf("EPSG:%s",keep$EPSG_NAD83)
  St.sub.pr <- spTransform(St.sub, CRS(SRS_string=sprintf("EPSG:%s",keep$EPSG_NAD83)))
  outname <- sprintf("%s_%s_buff_50km.gpkg",StatePostalCode, keep$EPSG_NAD83)
  writeOGR(St.sub.pr, dsn = outname, layer = sprintf("%s_%s_buff_50km",StatePostalCode, keep$EPSG_NAD83), 
           driver = "GPKG", overwrite_layer = TRUE)
  
  ### Re-project state boundary and save to same directory 
  St.bound <- subset(states.boundary, STUSPS == StatePostalCode)
  St.bound.pr <- spTransform(St.bound, CRS(SRS_string=sprintf("EPSG:%s",keep$EPSG_NAD83)))
  outname2 <- sprintf("%s_%s.gpkg",StatePostalCode, keep$EPSG_NAD83)
  writeOGR(St.bound.pr, dsn = outname2, layer = sprintf("%s_%s",StatePostalCode, keep$EPSG_NAD83), 
           driver = "GPKG", overwrite_layer = TRUE)
  
  #### Re-project NLCD layer based on selected SPCS/EPSG 
  NLCD.mask.sr <- terra::rast(NLCD.mask)
  NLCD.mask.sr[NLCD.mask.sr==0]<-NA #replace 0 (no data) with NA values
  NLCD.sr.pr <- terra::project(NLCD.mask.sr, a_crs, res=30, method="near")
  plot(NLCD.sr.pr)
  barplot(NLCD.sr.pr, xlab="NLCD projected")

  ### Write out the masked projected raster
  writeRaster(NLCD.sr.pr, filename=sprintf("NLCD2016_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT1U',overwrite=TRUE)
  
  ### Reclassify masked/projected NLCD layer to forest
  f <- c(0,NA,11,0,12,0,21,0,22,0,23,0,24,0,31,0,41,1,42,1,43,1,51,1,52,1,71,0,
         72,0,73,0,74,0,81,0,82,0,90,1,95,1)  ### forest = 41,42,43, 51,52, 90, and 95 
  rclmatf <- matrix(f, ncol=2, byrow=TRUE)
  rcf <- classify(NLCD.sr.pr, rclmatf)
  barplot(rcf, xlab="reclass forest") ### check to make sure it worked!
  writeRaster(rcf,filename=sprintf("NLCD2016_Forested_%s_%s.tif",StatePostalCode, keep$EPSG_NAD83), datatype='INT1U',overwrite=TRUE)
  
  ### Reclassify masked/projected NLCD layer to developed
  d <- c(0,NA,11,0,12,0,21,1,22,1,23,1,24,1,31,1,41,0,42,0,43,0,51,0,52,0,71,0,
         72,0,73,0,74,0,81,0,82,0,90,0,95,0)   ### developed = 21-24, 31 
  rclmatd <- matrix(d, ncol=2, byrow=TRUE)
  rcd <- classify(NLCD.sr.pr, rclmatd)
  barplot(rcd, xlab="reclass developed")
  writeRaster(rcd,filename=sprintf("NLCD2016_Developed_%s_%s.tif",StatePostalCode,keep$EPSG_NAD83), datatype='INT1U',overwrite=TRUE)
  
  ### Reclassify masked/projected NLCD layer to developed
  o <- c(0,NA,11,0,12,0,21,0,22,0,23,0,24,0,31,0,41,0,42,0,43,0,51,0,52,0,71,1,
         72,1,73,1,74,1,81,1,82,1,90,0,95,0)  ### open = 71-74, 81, 82 
  rclmato <- matrix(o, ncol=2, byrow=TRUE)
  rco <- classify(NLCD.sr.pr, rclmato)
  barplot(rco, xlab="reclass open")
  writeRaster(rco,filename=sprintf("NLCD2016_Open_%s_%s.tif",StatePostalCode,keep$EPSG_NAD83), datatype='INT1U',overwrite=TRUE)
  
  ### set working directory back to main folder
  setwd(wd)
}
  
### Replace XX below with the 2 letter state postal code (for example, NY, OH, VA). 
### The line calling the function can be duplicated to run the process sequentially for multiple states.

# Process_NLCD_layers(StatePostalCode="XX") 


